@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -180,9 +180,9 @@ ensure_nss(void *parent, const char *path, char **nss_scheme)
 		*nss_scheme = talloc_strdup(parent, "extern");
 		path += 7;
 	}
-	ret = cm_store_canonicalize_directory(parent, path);
+	ret = ensure_path_is_absolute(parent, path);
 	if (ret != NULL) {
-		ret = ensure_path_is_absolute(parent, ret);
+		ret = cm_store_canonicalize_directory(parent, ret);
 	}
 	if (ret != NULL) {
 		if (ensure_path_is_directory(ret) != 0) {
@@ -200,9 +200,9 @@ static char *
 ensure_pem(void *parent, const char *path)
 {
 	char *ret;
-	ret = cm_store_canonicalize_directory(parent, path);
+	ret = ensure_path_is_absolute(parent, path);
 	if (ret != NULL) {
-		ret = ensure_path_is_absolute(parent, ret);
+		ret = cm_store_canonicalize_directory(parent, ret);
 	}
 	if (ret != NULL) {
 		if (ensure_parent_is_directory(parent, ret) != 0) {
@@ -456,6 +456,88 @@ query_rep_sososos(enum cm_tdbus_type which,
 	dbus_message_unref(rep);
 }
 
+/* Send a query for the value of the specified property to the named object and
+ * return the reply message. */
+static DBusMessage *
+query_prop(enum cm_tdbus_type which,
+	   const char *path, const char *interface, const char *prop,
+	   int verbose)
+{
+	DBusMessage *req;
+	req = prep_req(which, path, DBUS_INTERFACE_PROPERTIES, "Get");
+	cm_tdbusm_set_ss(req, interface, prop);
+	return send_req(req, verbose);
+}
+
+/* Read a boolean property. */
+static dbus_bool_t
+query_prop_b(enum cm_tdbus_type which,
+	     const char *path, const char *interface, const char *prop,
+	     int verbose,
+	     void *parent)
+{
+	DBusMessage *rep;
+	dbus_bool_t b;
+	rep = query_prop(which, path, interface, prop, verbose);
+	if (cm_tdbusm_get_b(rep, parent, &b) != 0) {
+		printf(_("Error parsing server response.\n"));
+		exit(1);
+	}
+	dbus_message_unref(rep);
+	return b;
+}
+
+/* Read a string property. */
+static char *
+query_prop_s(enum cm_tdbus_type which,
+	     const char *path, const char *interface, const char *prop,
+	     int verbose,
+	     void *parent)
+{
+	DBusMessage *rep;
+	char *s;
+	rep = query_prop(which, path, interface, prop, verbose);
+	if (cm_tdbusm_get_s(rep, parent, &s) != 0) {
+		s = "";
+	}
+	dbus_message_unref(rep);
+	return s;
+}
+
+/* Read a path property. */
+static char *
+query_prop_p(enum cm_tdbus_type which,
+	     const char *path, const char *interface, const char *prop,
+	     int verbose,
+	     void *parent)
+{
+	DBusMessage *rep;
+	char *p;
+	rep = query_prop(which, path, interface, prop, verbose);
+	if (cm_tdbusm_get_p(rep, parent, &p) != 0) {
+		p = "";
+	}
+	dbus_message_unref(rep);
+	return p;
+}
+
+/* Read an array-of-strings property. */
+static char **
+query_prop_as(enum cm_tdbus_type which,
+	      const char *path, const char *interface, const char *prop,
+	      int verbose,
+	      void *parent)
+{
+	DBusMessage *rep;
+	char **as;
+	rep = query_prop(which, path, interface, prop, verbose);
+	if (cm_tdbusm_get_as(rep, parent, &as) != 0) {
+		as = NULL;
+	}
+	dbus_message_unref(rep);
+	return as;
+}
+
 /* Add a new request. */
 static int
 request(const char *argv0, int argc, char **argv)
@@ -468,15 +550,15 @@ request(const char *argv0, int argc, char **argv)
 	int keysize = 0, auto_renew = 1, verbose = 0, c, i;
 	char *ca = DEFAULT_CA, *subject = NULL, **eku = NULL, *oid, *id = NULL;
 	char **principal = NULL, **dns = NULL, **email = NULL;
-	struct cm_tdbusm_dict param[32];
-	const struct cm_tdbusm_dict *params[32];
+	struct cm_tdbusm_dict param[33];
+	const struct cm_tdbusm_dict *params[34];
 	DBusMessage *req, *rep;
 	dbus_bool_t b;
 	char *p;
 	krb5_context kctx;
 	krb5_error_code kret;
 	krb5_principal kprincipal;
-	char *krealm, *kuprincipal;
+	char *krealm, *kuprincipal, *command = NULL;
 
 	memset(subject_default, '\0', sizeof(subject_default));
 	strcpy(subject_default, "CN=");
@@ -498,8 +580,9 @@ request(const char *argv0, int argc, char **argv)
 		krealm = NULL;
 	}
 
+	opterr = 0;
 	while ((c = getopt(argc, argv,
-			   "d:n:t:k:f:I:g:rRN:U:K:D:E:sSp:P:v"
+			   ":d:n:t:k:f:I:g:rRN:U:K:D:E:sSp:P:vC:"
 			   GETOPT_CA)) != -1) {
 		switch (c) {
 		case 'd':
@@ -588,15 +671,31 @@ request(const char *argv0, int argc, char **argv)
 		case 'P':
 			pin = optarg;
 			break;
+		case 'C':
+			command = optarg;
+			break;
 		case 'v':
 			verbose++;
 			break;
 		default:
+			if (c == ':') {
+				fprintf(stderr,
+					_("%s: option requires an argument -- '%c'\n"),
+					"request", optopt);
+			} else {
+				fprintf(stderr,
+					_("%s: invalid option -- '%c'\n"),
+					"request", optopt);
+			}
 			help(argv0, "request");
 			return 1;
 		}
 	}
 	if (optind < argc) {
+		for (c = optind; c < argc; c++) {
+			printf(_("Error: unused extra argument \"%s\".\n"),
+			       argv[c]);
+		}
 		printf(_("Error: unused extra arguments were supplied.\n"));
 		help(argv0, "request");
 		return 1;
@@ -783,7 +882,7 @@ request(const char *argv0, int argc, char **argv)
 			return 1;
 		}
 		param[i].key = "CA";
-		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value_type = cm_tdbusm_dict_p;
 		param[i].value.s = capath;
 		params[i] = &param[i];
 		i++;
@@ -823,6 +922,13 @@ request(const char *argv0, int argc, char **argv)
 		params[i] = &param[i];
 		i++;
 	}
+	if (command != NULL) {
+		param[i].key = CM_DBUS_PROP_CERT_POSTSAVE_COMMAND;
+		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value.s = command;
+		params[i] = &param[i];
+		i++;
+	}
 	params[i] = NULL;
 	req = prep_req(bus, CM_DBUS_BASE_PATH, CM_DBUS_BASE_INTERFACE,
 		       "add_request");
@@ -980,13 +1086,13 @@ add_basic_request(enum cm_tdbus_type bus, char *id,
 		  char *dbdir, char *nickname, char *token,
 		  char *keyfile, char *certfile,
 		  char *pin, char *pinfile,
-		  char *ca, dbus_bool_t auto_renew_stop,
+		  char *ca, char *command, dbus_bool_t auto_renew_stop,
 		  int verbose)
 {
 	DBusMessage *req, *rep;
 	int i;
-	struct cm_tdbusm_dict param[19];
-	const struct cm_tdbusm_dict *params[20];
+	struct cm_tdbusm_dict param[20];
+	const struct cm_tdbusm_dict *params[21];
 	dbus_bool_t b;
 	const char *capath;
 	char *p;
@@ -1092,6 +1198,13 @@ add_basic_request(enum cm_tdbus_type bus, char *id,
 	param[i].value.b = !auto_renew_stop;
 	params[i] = &param[i];
 	i++;
+	if (command != NULL) {
+		param[i].key = CM_DBUS_PROP_CERT_POSTSAVE_COMMAND;
+		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value.s = command;
+		params[i] = &param[i];
+		i++;
+	}
 	if (ca != NULL) {
 		capath = find_ca_by_name(globals.tctx, bus, ca, verbose);
 		if (capath == NULL) {
@@ -1099,7 +1212,7 @@ add_basic_request(enum cm_tdbus_type bus, char *id,
 			return 1;
 		}
 		param[i].key = "CA";
-		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value_type = cm_tdbusm_dict_p;
 		param[i].value.s = talloc_strdup(globals.tctx, capath);
 		params[i] = &param[i];
 		i++;
@@ -1137,8 +1250,8 @@ set_tracking(const char *argv0, const char *category,
 	enum cm_tdbus_type bus = CM_DBUS_DEFAULT_BUS;
 	DBusMessage *req, *rep;
 	const char *request, *capath;
-	struct cm_tdbusm_dict param[10];
-	const struct cm_tdbusm_dict *params[11];
+	struct cm_tdbusm_dict param[11];
+	const struct cm_tdbusm_dict *params[12];
 	char *nss_scheme, *dbdir = NULL, *token = NULL, *nickname = NULL;
 	char *id = NULL, *new_id = NULL, *new_request;
 	char *keyfile = NULL, *certfile = NULL, *ca = DEFAULT_CA;
@@ -1151,6 +1264,7 @@ set_tracking(const char *argv0, const char *category,
 	krb5_error_code kret;
 	krb5_principal kprincipal;
 	char *krealm, *kuprincipal;
+	char *command = NULL;
 
 	kctx = NULL;
 	if ((kret = krb5_init_context(&kctx)) != 0) {
@@ -1164,8 +1278,9 @@ set_tracking(const char *argv0, const char *category,
 		krealm = NULL;
 	}
 
+	opterr = 0;
 	while ((c = getopt(argc, argv,
-			   "d:n:t:k:f:g:p:P:rRi:I:U:K:D:E:sSv"
+			   ":d:n:t:k:f:g:p:P:rRi:I:U:K:D:E:sSvC:"
 			   GETOPT_CA)) != -1) {
 		switch (c) {
 		case 'd':
@@ -1266,10 +1381,21 @@ set_tracking(const char *argv0, const char *category,
 		case 'P':
 			pin = optarg;
 			break;
+		case 'C':
+			command = optarg;
+			break;
 		case 'v':
 			verbose++;
 			break;
 		default:
+			if (c == ':') {
+				fprintf(stderr,
+					_("%s: option requires an argument -- '%c'\n"),
+					category, optopt);
+			} else {
+				fprintf(stderr, _("%s: invalid option -- '%c'\n"),
+					category, optopt);
+			}
 			help(argv0, category);
 			return 1;
 		}
@@ -1392,7 +1518,7 @@ set_tracking(const char *argv0, const char *category,
 					return 1;
 				}
 				param[i].key = "CA";
-				param[i].value_type = cm_tdbusm_dict_s;
+				param[i].value_type = cm_tdbusm_dict_p;
 				param[i].value.s = talloc_strdup(globals.tctx,
 								 capath);
 				params[i] = &param[i];
@@ -1400,6 +1526,13 @@ set_tracking(const char *argv0, const char *category,
 			} else {
 				capath = NULL;
 			}
+			if (command != NULL) {
+				param[i].key = CM_DBUS_PROP_CERT_POSTSAVE_COMMAND;
+				param[i].value_type = cm_tdbusm_dict_s;
+				param[i].value.s = command;
+				params[i] = &param[i];
+				i++;
+			}
 			params[i] = NULL;
 			req = prep_req(bus, request, CM_DBUS_REQUEST_INTERFACE,
 				       "modify");
@@ -1461,7 +1594,7 @@ set_tracking(const char *argv0, const char *category,
 						 dbdir, nickname, token,
 						 keyfile, certfile,
 						 pin, pinfile,
-						 ca, (auto_renew_stop > 0),
+						 ca, command, (auto_renew_stop > 0),
 						 verbose);
 		}
 	} else {
@@ -1522,9 +1655,10 @@ resubmit(const char *argv0, int argc, char **argv)
 {
 	enum cm_tdbus_type bus = CM_DBUS_DEFAULT_BUS;
 	DBusMessage *req, *rep;
-	const char *request, *capath;
-	struct cm_tdbusm_dict param[15];
-	const struct cm_tdbusm_dict *params[16];
+	const char *request;
+	char *capath;
+	struct cm_tdbusm_dict param[16];
+	const struct cm_tdbusm_dict *params[17];
 	char *dbdir = NULL, *token = NULL, *nickname = NULL, *certfile = NULL;
 	char *pin = NULL, *pinfile = NULL;
 	char *id = NULL, *new_id = NULL, *ca = NULL, *new_request, *nss_scheme;
@@ -1535,7 +1669,7 @@ resubmit(const char *argv0, int argc, char **argv)
 	krb5_context kctx;
 	krb5_error_code kret;
 	krb5_principal kprincipal;
-	char *kuprincipal;
+	char *kuprincipal, *command = NULL;
 
 	kctx = NULL;
 	if ((kret = krb5_init_context(&kctx)) != 0) {
@@ -1545,8 +1679,9 @@ resubmit(const char *argv0, int argc, char **argv)
 		return 1;
 	}
 
+	opterr = 0;
 	while ((c = getopt(argc, argv,
-			   "d:n:N:t:U:K:E:D:f:i:I:sSp:P:v" GETOPT_CA)) != -1) {
+			   ":d:n:N:t:U:K:E:D:f:i:I:sSp:P:vC:" GETOPT_CA)) != -1) {
 		switch (c) {
 		case 'd':
 			nss_scheme = NULL;
@@ -1625,10 +1760,21 @@ resubmit(const char *argv0, int argc, char **argv)
 		case 'P':
 			pin = optarg;
 			break;
+		case 'C':
+			command = optarg;
+			break;
 		case 'v':
 			verbose++;
 			break;
 		default:
+			if (c == ':') {
+				fprintf(stderr,
+					_("%s: option requires an argument -- '%c'\n"),
+					"resubmit", optopt);
+			} else {
+				fprintf(stderr, _("%s: invalid option -- '%c'\n"),
+					"resubmit", optopt);
+			}
 			help(argv0, "resubmit");
 			return 1;
 		}
@@ -1695,7 +1841,7 @@ resubmit(const char *argv0, int argc, char **argv)
 			exit(1);
 		}
 		param[i].key = "CA";
-		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value_type = cm_tdbusm_dict_p;
 		param[i].value.s = talloc_strdup(globals.tctx, capath);
 		params[i] = &param[i];
 		i++;
@@ -1749,6 +1895,13 @@ resubmit(const char *argv0, int argc, char **argv)
 		params[i] = &param[i];
 		i++;
 	}
+	if (command != NULL) {
+		param[i].key = CM_DBUS_PROP_CERT_POSTSAVE_COMMAND;
+		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value.s = command;
+		params[i] = &param[i];
+		i++;
+	}
 	params[i] = NULL;
 	if (i > 0) {
 		req = prep_req(bus, request, CM_DBUS_REQUEST_INTERFACE,
@@ -1773,9 +1926,9 @@ resubmit(const char *argv0, int argc, char **argv)
 			exit(1);
 		}
 	}
-	capath = query_rep_p(bus, request, CM_DBUS_REQUEST_INTERFACE,
-			     "get_ca", verbose, globals.tctx);
-	if (capath != NULL) {
+	rep = query_rep(bus, request,
+			CM_DBUS_REQUEST_INTERFACE, "get_ca", verbose);
+	if (cm_tdbusm_get_p(rep, globals.tctx, &capath) == 0) {
 		ca = find_ca_name(globals.tctx, bus, capath, verbose);
 	} else {
 		ca = NULL;
@@ -1818,7 +1971,9 @@ list(const char *argv0, int argc, char **argv)
 	long n1, n2;
 	char **as1, **as2, **as3, **as4, t[24];
 	int requests_only = 0, tracking_only = 0, verbose = 0, c, i, j;
-	while ((c = getopt(argc, argv, "rtsSvd:n:f:i:" GETOPT_CA)) != -1) {
+
+	opterr = 0;
+	while ((c = getopt(argc, argv, ":rtsSvd:n:f:i:" GETOPT_CA)) != -1) {
 		switch (c) {
 		case 'c':
 			only_ca = optarg;
@@ -1856,6 +2011,14 @@ list(const char *argv0, int argc, char **argv)
 			verbose++;
 			break;
 		default:
+			if (c == ':') {
+				fprintf(stderr,
+					_("%s: option requires an argument -- '%c'\n"),
+					"list", optopt);
+			} else {
+				fprintf(stderr, _("%s: invalid option -- '%c'\n"),
+					"list", optopt);
+			}
 			help(argv0, "list");
 			return 1;
 		}
@@ -1951,6 +2114,7 @@ list(const char *argv0, int argc, char **argv)
 		case CM_NEED_TO_SAVE_CERT:
 		case CM_SAVING_CERT:
 		case CM_SAVED_CERT:
+		case CM_POST_SAVED_CERT:
 		case CM_NEED_TO_READ_CERT:
 		case CM_READING_CERT:
 		case CM_CA_WORKING:
@@ -2118,6 +2282,9 @@ list(const char *argv0, int argc, char **argv)
 			       cm_oid_to_name(NULL, as4[j]),
 			       as4[j + 1] ? "" : "\n");
 		}
+		printf(_("\tcommand: %s\n"),
+		       query_prop_s(bus, requests[i], CM_DBUS_REQUEST_INTERFACE,
+				    CM_DBUS_PROP_CERT_POSTSAVE_COMMAND, verbose, globals.tctx));
 		printf(_("\ttrack: %s\n"),
 		       query_rep_b(bus, requests[i], CM_DBUS_REQUEST_INTERFACE,
 				   "get_monitoring", verbose, globals.tctx) ?
@@ -2137,7 +2304,9 @@ list_cas(const char *argv0, int argc, char **argv)
 	char **cas, *s, *only_ca = DEFAULT_CA;
 	char **as;
 	int c, i, j, verbose = 0;
-	while ((c = getopt(argc, argv, "sSv" GETOPT_CA)) != -1) {
+
+	opterr = 0;
+	while ((c = getopt(argc, argv, ":sSv" GETOPT_CA)) != -1) {
 		switch (c) {
 		case 'c':
 			only_ca = optarg;
@@ -2152,6 +2321,14 @@ list_cas(const char *argv0, int argc, char **argv)
 			verbose++;
 			break;
 		default:
+			if (c == ':') {
+				fprintf(stderr,
+					_("%s: option requires an argument -- '%c'\n"),
+					"list-cas", optopt);
+			} else {
+				fprintf(stderr, _("%s: invalid option -- '%c'\n"),
+					"list-cas", optopt);
+			}
 			help(argv0, "list-cas");
 			return 1;
 		}
@@ -2258,6 +2435,7 @@ help(const char *cmd, const char *category)
 		N_("  -S		connect to the certmonger service on the system bus\n"),
 		N_("  -s		connect to the certmonger service on the session bus\n"),
 		N_("* Other options:\n"),
+		N_("  -C	command to run when saving the certificate\n"),
 		N_("  -v	report all details of errors\n"),
 		NULL,
 	};
@@ -2295,6 +2473,7 @@ help(const char *cmd, const char *category)
 		N_("  -S		connect to the certmonger service on the system bus\n"),
 		N_("  -s		connect to the certmonger service on the session bus\n"),
 		N_("* Other options:\n"),
+		N_("  -C	command to run when saving the certificate\n"),
 		N_("  -v	report all details of errors\n"),
 		NULL,
 	};
@@ -2354,6 +2533,7 @@ help(const char *cmd, const char *category)
 		N_("  -S		connect to the certmonger service on the system bus\n"),
 		N_("  -s		connect to the certmonger service on the session bus\n"),
 		N_("* Other options:\n"),
+		N_("  -C	command to run when saving the certificate\n"),
 		N_("  -v	report all details of errors\n"),
 		NULL,
 	};
