@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include "keyiread.h"
 #include "log.h"
 #include "notify.h"
+#include "postsave.h"
 #include "prefs.h"
 #include "store.h"
 #include "store-int.h"
@@ -47,6 +48,7 @@ struct cm_iterate_state {
 	struct cm_csrgen_state *cm_csrgen_state;
 	struct cm_submit_state *cm_submit_state;
 	struct cm_certsave_state *cm_certsave_state;
+	struct cm_postsave_state *cm_postsave_state;
 	struct cm_certread_state *cm_certread_state;
 	struct cm_notify_state *cm_notify_state;
 };
@@ -115,6 +117,9 @@ cm_entry_reset_state(struct cm_store_entry *entry)
 		break;
 	case CM_SAVED_CERT:
 		break;
+	case CM_POST_SAVED_CERT:
+		entry->cm_state = CM_POST_SAVED_CERT;
+		break;
 	case CM_CA_REJECTED:
 		break;
 	case CM_CA_WORKING:
@@ -248,7 +253,8 @@ cm_iterate_init(struct cm_store_entry *entry, void **cm_iterate_state)
 		state->cm_certread_state = NULL;
 	}
 	cm_store_entry_save(entry);
-	cm_log(3, "'%s' starts in state '%s'\n", entry->cm_id,
+	cm_log(3, "%s('%s') starts in state '%s'\n",
+	       entry->cm_busname, entry->cm_nickname,
 	       cm_store_state_as_string(entry->cm_state));
 	return 0;
 }
@@ -313,6 +319,11 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 	   struct cm_context *context,
 	   struct cm_store_ca *(*get_ca_by_index)(struct cm_context *, int),
 	   int (*get_n_cas)(struct cm_context *),
+	   void (*emit_entry_saved_cert)(struct cm_context *,
+					 struct cm_store_entry *),
+	   void (*emit_entry_changes)(struct cm_context *,
+				      struct cm_store_entry *,
+				      struct cm_store_entry *),
 	   void *cm_iterate_state,
 	   enum cm_time *when, int *delay, int *readfd)
 {
@@ -320,7 +331,7 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 	time_t remaining;
 	struct cm_iterate_state *state;
 	struct cm_store_ca *tmp_ca;
-	enum cm_state old_entry_state;
+	struct cm_store_entry *old_entry;
 	char *serial;
 	const char *tmp_ca_name;
 	state = cm_iterate_state;
@@ -328,7 +339,7 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 	*when = cm_time_no_time;
 	*delay = 0;
 
-	old_entry_state = entry->cm_state;
+	old_entry = cm_store_entry_dup(entry, entry);
 	if (entry->cm_cert_not_after != 0) {
 		remaining = entry->cm_cert_not_after - cm_time(NULL);
 	} else {
@@ -627,9 +638,11 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 				cm_submit_done(entry, state->cm_submit_state);
 				state->cm_submit_state = NULL;
 				if (entry->cm_cert != NULL) {
-					cm_log(3, "'%s' already had a "
+					cm_log(3, "%s('%s') already had a "
 					       "certificate, going back to "
-					       "monitoring it\n", entry->cm_id);
+					       "monitoring it\n",
+					       entry->cm_busname,
+					       entry->cm_nickname);
 					entry->cm_state = CM_MONITORING;
 					*when = cm_time_soonish;
 				} else {
@@ -663,9 +676,11 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 				cm_submit_done(entry, state->cm_submit_state);
 				state->cm_submit_state = NULL;
 				if (entry->cm_cert != NULL) {
-					cm_log(3, "'%s' already had a "
+					cm_log(3, "%s('%s') already had a "
 					       "certificate, going back to "
-					       "monitoring it\n", entry->cm_id);
+					       "monitoring it\n",
+					       entry->cm_busname,
+					       entry->cm_nickname);
 					entry->cm_state = CM_MONITORING;
 					*when = cm_time_soonish;
 				} else {
@@ -677,7 +692,9 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 				/* Don't know what's going on. HELP! */
 				cm_log(1,
 				       "Unable to determine course of action "
-				       "for \"%s\".\n", entry->cm_id);
+				       "for %s('%s').\n",
+				       entry->cm_busname,
+				       entry->cm_nickname);
 				cm_submit_done(entry, state->cm_submit_state);
 				state->cm_submit_state = NULL;
 				entry->cm_state = CM_NEED_GUIDANCE;
@@ -771,6 +788,9 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 			state->cm_certread_state = NULL;
 			entry->cm_state = CM_SAVED_CERT;
 			*when = cm_time_now;
+			if (emit_entry_saved_cert != NULL) {
+				(*emit_entry_saved_cert)(context, entry);
+			}
 		} else {
 			/* Wait for status update, or poll. */
 			*readfd = cm_certread_get_fd(entry,
@@ -784,8 +804,46 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		break;
 
 	case CM_SAVED_CERT:
-		entry->cm_state = CM_MONITORING;
-		*when = cm_time_now;
+		if (entry->cm_post_certsave_command != NULL) {
+			state->cm_postsave_state = cm_postsave_start(entry);
+			if (state->cm_postsave_state != NULL) {
+				/* Note that we're doing the post-save. */
+				entry->cm_state = CM_POST_SAVED_CERT;
+				/* Wait for status update, or poll. */
+				*readfd = cm_postsave_get_fd(entry,
+							     state->cm_postsave_state);
+				if (*readfd == -1) {
+					*when = cm_time_soon;
+				} else {
+					*when = cm_time_no_time;
+				}
+			} else {
+				/* Failed to start the post-save; skip it. */
+				entry->cm_state = CM_MONITORING;
+				*when = cm_time_soon;
+			}
+		} else {
+			entry->cm_state = CM_MONITORING;
+			*when = cm_time_now;
+		}
+		break;
+
+	case CM_POST_SAVED_CERT:
+		if (cm_postsave_ready(entry, state->cm_postsave_state) == 0) {
+			cm_postsave_done(entry, state->cm_postsave_state);
+			state->cm_postsave_state = NULL;
+			entry->cm_state = CM_MONITORING;
+			*when = cm_time_now;
+		} else {
+			/* Wait for status update, or poll. */
+			*readfd = cm_postsave_get_fd(entry,
+						     state->cm_postsave_state);
+			if (*readfd == -1) {
+				*when = cm_time_soon;
+			} else {
+				*when = cm_time_no_time;
+			}
+		}
 		break;
 
 	case CM_CA_REJECTED:
@@ -999,7 +1057,7 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 	case CM_NEWLY_ADDED_DECIDING:
 		/* Decide what to do next.  Assign a CA if it doesn't have one
 		 * assigned to it already. */
-		if ((entry->cm_ca_name == NULL) &&
+		if ((entry->cm_ca_nickname == NULL) &&
 		    (entry->cm_cert_issuer != NULL)) {
 			/* Walk the list of known names of known CAs and try to
 			 * match one with the issuer of the certificate we
@@ -1012,26 +1070,26 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 				     j++) {
 					if (strcmp(tmp_ca->cm_ca_known_issuer_names[j],
 						   entry->cm_cert_issuer) == 0) {
-						entry->cm_ca_name = talloc_strdup(entry, tmp_ca->cm_id);
+						entry->cm_ca_nickname = talloc_strdup(entry, tmp_ca->cm_nickname);
 					}
 				}
 			}
 		}
 		/* No match -> assign the default. */
-		if (entry->cm_ca_name == NULL) {
+		if (entry->cm_ca_nickname == NULL) {
 			for (i = 0; i < (*get_n_cas)(context); i++) {
 				tmp_ca = (*get_ca_by_index)(context, i);
 				if (tmp_ca->cm_ca_is_default) {
-					entry->cm_ca_name = talloc_strdup(entry, tmp_ca->cm_id);
+					entry->cm_ca_nickname = talloc_strdup(entry, tmp_ca->cm_nickname);
 				}
 			}
 		}
 		/* No default in our data store -> use the config file's. */
-		if (entry->cm_ca_name == NULL) {
+		if (entry->cm_ca_nickname == NULL) {
 			tmp_ca_name = cm_prefs_default_ca();
 			if (tmp_ca_name != NULL) {
-				entry->cm_ca_name = talloc_strdup(entry,
-								  tmp_ca_name);
+				entry->cm_ca_nickname = talloc_strdup(entry,
+								      tmp_ca_name);
 			}
 		}
 		/* If we have a certificate, we go straight to monitoring it.
@@ -1057,8 +1115,8 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 			cm_store_set_if_not_set_s(entry,
 						  &entry->cm_template_eku,
 						  entry->cm_cert_eku);
-			cm_log(3, "'%s' has a certificate, monitoring it\n",
-			       entry->cm_id);
+			cm_log(3, "%s('%s') has a certificate, monitoring it\n",
+			       entry->cm_busname, entry->cm_nickname);
 			entry->cm_state = CM_MONITORING;
 			*when = cm_time_now;
 		} else
@@ -1068,26 +1126,28 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 			/* If we don't have a certificate, but we have a key,
 			 * the next step is to generate a CSR. */
 			if (entry->cm_key_type.cm_key_size > 0) {
-				cm_log(3, "'%s' has no certificate, will "
+				cm_log(3, "%s('%s') has no certificate, will "
 				       "attempt enrollment using "
-				       "already-present key\n", entry->cm_id);
+				       "already-present key\n",
+				       entry->cm_busname, entry->cm_nickname);
 				entry->cm_state = CM_NEED_CSR;
 				*when = cm_time_now;
 			} else {
 				/* No certificate, no key, start with
 				 * generating the key. */
-				cm_log(3, "'%s' has no key or certificate, "
+				cm_log(3, "%s('%s') has no key or certificate, "
 				       "will generate keys and attempt "
-				       "enrollment\n", entry->cm_id);
+				       "enrollment\n",
+				       entry->cm_busname, entry->cm_nickname);
 				entry->cm_state = CM_NEED_KEY_PAIR;
 				*when = cm_time_now;
 			}
 		} else {
 			/* And if we don't have a place for the key, we're
 			 * screwed.  Hopefully this didn't happen normally. */
-			cm_log(3, "'%s' has no key or certificate location, "
-			       "don't know what to do about that\n",
-			       entry->cm_id);
+			cm_log(3, "%s('%s') has no key or certificate location,"
+			       " don't know what to do about that\n",
+			       entry->cm_busname, entry->cm_nickname);
 			entry->cm_state = CM_NEED_GUIDANCE;
 			*when = cm_time_now;
 		}
@@ -1098,12 +1158,18 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		abort();
 		break;
 	}
-	if (old_entry_state != entry->cm_state) {
-		cm_log(3, "'%s' moved to state '%s'\n",
-		       entry->cm_id ? entry->cm_id : "(unnamed entry)",
+	if (old_entry->cm_state != entry->cm_state) {
+		cm_log(3, "%s('%s') moved to state '%s'\n",
+		       entry->cm_busname,
+		       entry->cm_nickname ?
+		       entry->cm_nickname : "(unnamed entry)",
 		       cm_store_state_as_string(entry->cm_state));
 		cm_store_entry_save(entry);
 	}
+	if (emit_entry_changes != NULL) {
+		(*emit_entry_changes)(context, old_entry, entry);
+	}
+	talloc_free(old_entry);
 	return 0;
 }
 
@@ -1137,7 +1203,8 @@ cm_iterate_done(struct cm_store_entry *entry, void *cm_iterate_state)
 		talloc_free(state);
 	}
 	cm_entry_reset_state(entry);
-	cm_log(3, "'%s' ends in state '%s'\n", entry->cm_id,
+	cm_log(3, "%s('%s') ends in state '%s'\n",
+	       entry->cm_busname, entry->cm_nickname,
 	       cm_store_state_as_string(entry->cm_state));
 	return 0;
 }
