@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,8 @@
 #include "log.h"
 #include "tm.h"
 
+static unsigned long long cm_entry_name_last, cm_ca_name_last;
+
 enum cm_store_file_field {
 	cm_store_file_field_invalid = 0,
 	cm_store_file_field_id,
@@ -54,6 +56,7 @@ enum cm_store_file_field {
 	cm_store_entry_field_key_nickname,
 	cm_store_entry_field_key_pin,
 	cm_store_entry_field_key_pin_file,
+	cm_store_entry_field_key_pubkey,
 
 	cm_store_entry_field_cert_storage_type,
 	cm_store_entry_field_cert_storage_location,
@@ -91,7 +94,7 @@ enum cm_store_file_field {
 	cm_store_entry_field_autorenew,
 	cm_store_entry_field_monitor,
 
-	cm_store_entry_field_ca_name,
+	cm_store_entry_field_ca_nickname,
 
 	cm_store_entry_field_submitted,
 	cm_store_entry_field_ca_cookie,
@@ -99,6 +102,9 @@ enum cm_store_file_field {
 
 	cm_store_entry_field_cert,
 
+	cm_store_entry_field_post_certsave_command,
+	cm_store_entry_field_post_certsave_uid,
+
 	cm_store_ca_field_known_issuer_names,
 	cm_store_ca_field_is_default,
 
@@ -113,7 +119,8 @@ static struct cm_store_file_field_list {
 	enum cm_store_file_field field;
 	const char *name;
 } cm_store_file_field_list[] = {
-	{cm_store_file_field_id, "id"},
+	{cm_store_file_field_id, "id"}, /* ipa-client-install assumes that we'll
+					 * never rename this, so now we can't */
 	{cm_store_entry_field_key_type, "key_type"},
 	{cm_store_entry_field_key_gen_type, "key_gen_type"},
 	{cm_store_entry_field_key_size, "key_size"},
@@ -125,6 +132,7 @@ static struct cm_store_file_field_list {
 	{cm_store_entry_field_key_nickname, "key_nickname"},
 	{cm_store_entry_field_key_pin, "key_pin"},
 	{cm_store_entry_field_key_pin_file, "key_pin_file"},
+	{cm_store_entry_field_key_pubkey, "key_pubkey"},
 
 	{cm_store_entry_field_cert_storage_type, "cert_storage_type"},
 	{cm_store_entry_field_cert_storage_location, "cert_storage_location"},
@@ -163,13 +171,15 @@ static struct cm_store_file_field_list {
 	{cm_store_entry_field_autorenew, "autorenew"},
 	{cm_store_entry_field_monitor, "monitor"},
 
-	{cm_store_entry_field_ca_name, "ca_name"},
+	{cm_store_entry_field_ca_nickname, "ca_name"},
 
 	{cm_store_entry_field_submitted, "submitted"},
 	{cm_store_entry_field_ca_cookie, "ca_cookie"},
 	{cm_store_entry_field_ca_error, "ca_error"},
 
 	{cm_store_entry_field_cert, "cert"},
+	{cm_store_entry_field_post_certsave_command, "post_certsave_command"},
+	{cm_store_entry_field_post_certsave_uid, "post_certsave_uid"},
 
 	{cm_store_ca_field_known_issuer_names, "ca_issuer_names"},
 	{cm_store_ca_field_is_default, "ca_is_default"},
@@ -357,6 +367,12 @@ free_if_empty_multi(void *parent, char *p)
 	return s;
 }
 
+char *
+cm_store_entry_next_busname(void *parent)
+{
+	return talloc_asprintf(parent, "Request%llu", ++cm_entry_name_last);
+}
+
 static struct cm_store_entry *
 cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 {
@@ -368,6 +384,7 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 	if (ret != NULL) {
 		memset(ret, 0, sizeof(*ret));
 		s = cm_store_file_read_lines(ret, fp);
+		ret->cm_busname = cm_store_entry_next_busname(ret);
 		ret->cm_store_private = talloc_strdup(ret, filename);
 		for (i = 0; (s != NULL) && (s[i] != NULL); i++) {
 			p = s[i];
@@ -384,7 +401,7 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_ca_field_external_helper:
 				break;
 			case cm_store_file_field_id:
-				ret->cm_id = free_if_empty(p);
+				ret->cm_nickname = free_if_empty(p);
 				break;
 			case cm_store_entry_field_key_type:
 				if (strcasecmp(s[i], "RSA") == 0) {
@@ -449,9 +466,18 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 				break;
 			case cm_store_entry_field_key_pin:
 				ret->cm_key_pin = free_if_empty(p);
+				if (ret->cm_key_pin_file != NULL) {
+					ret->cm_key_pin = NULL;
+				}
 				break;
 			case cm_store_entry_field_key_pin_file:
 				ret->cm_key_pin_file = free_if_empty(p);
+				if (ret->cm_key_pin_file != NULL) {
+					ret->cm_key_pin = NULL;
+				}
+				break;
+			case cm_store_entry_field_key_pubkey:
+				ret->cm_key_pubkey = free_if_empty(p);
 				break;
 			case cm_store_entry_field_cert_storage_type:
 				if (strcasecmp(p, "FILE") == 0) {
@@ -569,8 +595,8 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 				ret->cm_monitor = atoi(p);
 				talloc_free(p);
 				break;
-			case cm_store_entry_field_ca_name:
-				ret->cm_ca_name = free_if_empty(p);
+			case cm_store_entry_field_ca_nickname:
+				ret->cm_ca_nickname = free_if_empty(p);
 				break;
 			case cm_store_entry_field_submitted:
 				ret->cm_submitted =
@@ -586,6 +612,12 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_cert:
 				ret->cm_cert = free_if_empty(p);
 				break;
+			case cm_store_entry_field_post_certsave_command:
+				ret->cm_post_certsave_command  = free_if_empty(p);
+				break;
+			case cm_store_entry_field_post_certsave_uid:
+				ret->cm_post_certsave_uid = free_if_empty(p);
+				break;
 			}
 		}
 	}
@@ -597,16 +629,26 @@ cm_store_files_entry_read(void *parent, const char *filename)
 {
 	FILE *fp;
 	struct cm_store_entry *ret;
-	fp = fopen(filename, "r");
-	if (fp != NULL) {
-		ret = cm_store_entry_read(parent, filename, fp);
-		fclose(fp);
+	if (filename != NULL) {
+		fp = fopen(filename, "r");
+		if (fp != NULL) {
+			ret = cm_store_entry_read(parent, filename, fp);
+			fclose(fp);
+		} else {
+			ret = NULL;
+		}
 	} else {
 		ret = NULL;
 	}
 	return ret;
 }
 
+char *
+cm_store_ca_next_busname(void *parent)
+{
+	return talloc_asprintf(parent, "CA%llu", ++cm_ca_name_last);
+}
+
 static struct cm_store_ca *
 cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 {
@@ -618,6 +660,7 @@ cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 	if (ret != NULL) {
 		memset(ret, 0, sizeof(*ret));
 		s = cm_store_file_read_lines(ret, fp);
+		ret->cm_busname = cm_store_ca_next_busname(ret);
 		ret->cm_store_private = talloc_strdup(ret, filename);
 		for (i = 0; (s != NULL) && (s[i] != NULL); i++) {
 			p = s[i];
@@ -636,6 +679,7 @@ cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_key_nickname:
 			case cm_store_entry_field_key_pin:
 			case cm_store_entry_field_key_pin_file:
+			case cm_store_entry_field_key_pubkey:
 			case cm_store_entry_field_cert_storage_type:
 			case cm_store_entry_field_cert_storage_location:
 			case cm_store_entry_field_cert_token:
@@ -664,14 +708,16 @@ cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_state:
 			case cm_store_entry_field_autorenew:
 			case cm_store_entry_field_monitor:
-			case cm_store_entry_field_ca_name:
+			case cm_store_entry_field_ca_nickname:
 			case cm_store_entry_field_submitted:
 			case cm_store_entry_field_ca_cookie:
 			case cm_store_entry_field_ca_error:
 			case cm_store_entry_field_cert:
+			case cm_store_entry_field_post_certsave_command:
+			case cm_store_entry_field_post_certsave_uid:
 				break;
 			case cm_store_file_field_id:
-				ret->cm_id = free_if_empty(p);
+				ret->cm_nickname = free_if_empty(p);
 				break;
 			case cm_store_ca_field_known_issuer_names:
 				ret->cm_ca_known_issuer_names =
@@ -717,10 +763,14 @@ cm_store_files_ca_read(void *parent, const char *filename)
 {
 	FILE *fp;
 	struct cm_store_ca *ret;
-	fp = fopen(filename, "r");
-	if (fp != NULL) {
-		ret = cm_store_ca_read(parent, filename, fp);
-		fclose(fp);
+	if (filename != NULL) {
+		fp = fopen(filename, "r");
+		if (fp != NULL) {
+			ret = cm_store_ca_read(parent, filename, fp);
+			fclose(fp);
+		} else {
+			ret = NULL;
+		}
 	} else {
 		ret = NULL;
 	}
@@ -797,10 +847,10 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 	char timestamp[15];
 	const char *p;
 
-	if (entry->cm_id == NULL) {
+	if (entry->cm_nickname == NULL) {
 		p = cm_store_timestamp_from_time(cm_time(NULL), timestamp);
 	} else {
-		p = entry->cm_id;
+		p = entry->cm_nickname;
 	}
 	cm_store_file_write_str(fp, cm_store_file_field_id, p);
 
@@ -852,10 +902,14 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 				entry->cm_key_token);
 	cm_store_file_write_str(fp, cm_store_entry_field_key_nickname,
 				entry->cm_key_nickname);
-	cm_store_file_write_str(fp, cm_store_entry_field_key_pin,
-				entry->cm_key_pin);
+	if (entry->cm_key_pin_file == NULL) {
+		cm_store_file_write_str(fp, cm_store_entry_field_key_pin,
+					entry->cm_key_pin);
+	}
 	cm_store_file_write_str(fp, cm_store_entry_field_key_pin_file,
 				entry->cm_key_pin_file);
+	cm_store_file_write_str(fp, cm_store_entry_field_key_pubkey,
+				entry->cm_key_pubkey);
 
 	switch (entry->cm_cert_storage_type) {
 	case cm_cert_storage_file:
@@ -938,8 +992,8 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 	cm_store_file_write_int(fp, cm_store_entry_field_monitor,
 				entry->cm_monitor);
 
-	cm_store_file_write_str(fp, cm_store_entry_field_ca_name,
-				entry->cm_ca_name);
+	cm_store_file_write_str(fp, cm_store_entry_field_ca_nickname,
+				entry->cm_ca_nickname);
 	cm_store_file_write_str(fp, cm_store_entry_field_submitted,
 				cm_store_timestamp_from_time(entry->cm_submitted,
 							      timestamp));
@@ -948,6 +1002,10 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 	cm_store_file_write_str(fp, cm_store_entry_field_ca_error,
 				entry->cm_ca_error);
 	cm_store_file_write_str(fp, cm_store_entry_field_cert, entry->cm_cert);
+	cm_store_file_write_str(fp, cm_store_entry_field_post_certsave_command,
+				entry->cm_post_certsave_command);
+	cm_store_file_write_str(fp, cm_store_entry_field_post_certsave_uid,
+				entry->cm_post_certsave_uid);
 	if (ferror(fp)) {
 		return -1;
 	}
@@ -973,7 +1031,7 @@ cm_store_entry_delete(struct cm_store_entry *entry)
 		}
 	} else {
 		cm_log(3, "No file to remove for \"%s\".\n",
-		       entry->cm_id);
+		       entry->cm_nickname);
 		ret = 0;
 	}
 	return 0;
@@ -983,19 +1041,19 @@ static void
 cm_store_create_containing_dir(const char *path, int mode)
 {
 	char dir[PATH_MAX];
-	struct stat st;
 	int i;
 	if (strlen(path) >= sizeof(dir)) {
 		return;
 	}
 	for (i = 0, dir[0] = '\0'; path[i] != '\0'; i++) {
-		if ((i > 0) && (path[i] == '/')) { /* XXX */
-			if ((stat(dir, &st) == -1) && (errno == ENOENT)) {
-				if (mkdir(dir, mode) == -1) {
+		if ((i > 0) && (path[i] == '/')) {
+			if (mkdir(dir, mode) == -1) {
+				if (errno != EEXIST) {
 					cm_log(1, "Failed to create \"%s\": "
 					       "%s.\n", dir, strerror(errno));
 					break;
 				}
+			} else {
 				cm_log(2, "Created \"%s\".\n", dir);
 			}
 		}
@@ -1009,21 +1067,24 @@ cm_store_entry_save(struct cm_store_entry *entry)
 {
 	FILE *fp;
 	char timestamp[15], path[PATH_MAX];
-	int i, fd, give_up;
+	int i, fd = -1, give_up;
 	const char *directory;
 
 	if (entry->cm_store_private == NULL) {
 		cm_store_timestamp_from_time(cm_time(NULL), timestamp);
 		directory = cm_env_request_dir();
-		snprintf(path, sizeof(path), "%s/%s", directory, timestamp);
-		fd = open(path,
-			  O_WRONLY | O_CREAT | O_EXCL,
-			  S_IRUSR | S_IWUSR);
-		if ((fd == -1) && (errno == ENOENT)) {
-			cm_store_create_containing_dir(path, S_IRWXU);
+		if (directory != NULL) {
+			snprintf(path, sizeof(path), "%s/%s",
+				 directory, timestamp);
 			fd = open(path,
 				  O_WRONLY | O_CREAT | O_EXCL,
 				  S_IRUSR | S_IWUSR);
+			if ((fd == -1) && (errno == ENOENT)) {
+				cm_store_create_containing_dir(path, S_IRWXU);
+				fd = open(path,
+					  O_WRONLY | O_CREAT | O_EXCL,
+					  S_IRUSR | S_IWUSR);
+			}
 		}
 		if (fd == -1) {
 			switch (errno) {
@@ -1112,8 +1173,8 @@ cm_store_get_all_entries(void *parent)
 					if (ret[j] != NULL) {
 						/* Check for duplicate names. */
 						for (k = 0; k < j; k++) {
-							if (strcmp(ret[k]->cm_id,
-								   ret[j]->cm_id) == 0) {
+							if (strcmp(ret[k]->cm_nickname,
+								   ret[j]->cm_nickname) == 0) {
 								cm_store_entry_delete(ret[j]);
 								talloc_free(ret[j]);
 								ret[j] = NULL;
@@ -1140,10 +1201,10 @@ cm_store_ca_write(FILE *fp, struct cm_store_ca *ca)
 	const char *p;
 	char timestamp[15];
 
-	if (ca->cm_id == NULL) {
+	if (ca->cm_nickname == NULL) {
 		p = cm_store_timestamp_from_time(cm_time(NULL), timestamp);
 	} else {
-		p = ca->cm_id;
+		p = ca->cm_nickname;
 	}
 	cm_store_file_write_str(fp, cm_store_file_field_id, p);
 	cm_store_file_write_strs(fp,
@@ -1193,7 +1254,7 @@ cm_store_ca_delete(struct cm_store_ca *ca)
 			       filename, strerror(errno));
 		}
 	} else {
-		cm_log(3, "No file to remove for \"%s\".\n", ca->cm_id);
+		cm_log(3, "No file to remove for \"%s\".\n", ca->cm_nickname);
 		ret = 0;
 	}
 	return 0;
@@ -1204,49 +1265,51 @@ cm_store_ca_save(struct cm_store_ca *ca)
 {
 	FILE *fp;
 	char timestamp[15], path[PATH_MAX];
-	int i, fd, give_up;
+	int i, fd = -1, give_up;
 	const char *directory;
 
 	if (ca->cm_store_private == NULL) {
 		cm_store_timestamp_from_time(cm_time(NULL), timestamp);
 		directory = cm_env_ca_dir();
-		snprintf(path, sizeof(path), "%s/%s", directory, timestamp);
-		fd = open(path,
-			  O_WRONLY | O_CREAT | O_EXCL,
-			  S_IRUSR | S_IWUSR);
-		if ((fd == -1) && (errno == ENOENT)) {
-			cm_store_create_containing_dir(path, S_IRWXU);
+		if (directory != NULL) {
+			snprintf(path, sizeof(path), "%s/%s", directory, timestamp);
 			fd = open(path,
 				  O_WRONLY | O_CREAT | O_EXCL,
 				  S_IRUSR | S_IWUSR);
-		}
-		if (fd == -1) {
-			switch (errno) {
-			case ENOENT:
-			case EPERM:
-			case EACCES:
-				break;
-			default:
-				for (give_up = 0, i = 1;
-				     !give_up && (i < 1024);
-				     i++) {
-					snprintf(path, sizeof(path), "%s/%s-%d",
-						 directory, timestamp, i);
-					fd = open(path,
-						  O_WRONLY | O_CREAT | O_EXCL,
-						  S_IRUSR | S_IWUSR);
-					if (fd != -1) {
-						break;
-					}
-					switch (errno) {
-					case ENOENT:
-					case EPERM:
-					case EACCES:
-						give_up++;
-						break;
+			if ((fd == -1) && (errno == ENOENT)) {
+				cm_store_create_containing_dir(path, S_IRWXU);
+				fd = open(path,
+					  O_WRONLY | O_CREAT | O_EXCL,
+					  S_IRUSR | S_IWUSR);
+			}
+			if (fd == -1) {
+				switch (errno) {
+				case ENOENT:
+				case EPERM:
+				case EACCES:
+					break;
+				default:
+					for (give_up = 0, i = 1;
+					     !give_up && (i < 1024);
+					     i++) {
+						snprintf(path, sizeof(path), "%s/%s-%d",
+							 directory, timestamp, i);
+						fd = open(path,
+							  O_WRONLY | O_CREAT | O_EXCL,
+							  S_IRUSR | S_IWUSR);
+						if (fd != -1) {
+							break;
+						}
+						switch (errno) {
+						case ENOENT:
+						case EPERM:
+						case EACCES:
+							give_up++;
+							break;
+						}
 					}
+					break;
 				}
-				break;
 			}
 		}
 		if (fd == -1) {
@@ -1308,8 +1371,8 @@ cm_store_get_all_cas(void *parent)
 				if (ret[j] != NULL) {
 					/* Check for duplicate names. */
 					for (k = 0; k < j; k++) {
-						if (strcmp(ret[k]->cm_id,
-							   ret[j]->cm_id) == 0) {
+						if (strcmp(ret[k]->cm_nickname,
+							   ret[j]->cm_nickname) == 0) {
 							cm_store_ca_delete(ret[j]);
 							talloc_free(ret[j]);
 							ret[j] = NULL;
@@ -1331,8 +1394,9 @@ cm_store_get_all_cas(void *parent)
 		}
 		if (k == j) {
 			ret[j] = cm_store_ca_new(ret);
-			ret[j]->cm_id = talloc_strdup(ret[j],
-						      CM_SELF_SIGN_CA_NAME);
+			ret[j]->cm_busname = cm_store_ca_next_busname(ret[j]);
+			ret[j]->cm_nickname = talloc_strdup(ret[j],
+							    CM_SELF_SIGN_CA_NAME);
 			ret[j]->cm_ca_type = cm_ca_internal_self;
 			ret[j]->cm_ca_internal_serial = talloc_strdup(ret[j],
 								      CM_DEFAULT_CERT_SERIAL);
@@ -1342,13 +1406,16 @@ cm_store_get_all_cas(void *parent)
 		/* Make sure we get at least one IPA entry. */
 		for (k = 0; k < j; k++) {
 			if ((ret[k]->cm_ca_type == cm_ca_external) &&
-			    (strcmp(ret[k]->cm_id, CM_IPA_CA_NAME) == 0)) {
+			    (strcmp(ret[k]->cm_nickname,
+				    CM_IPA_CA_NAME) == 0)) {
 				break;
 			}
 		}
 		if (k == j) {
 			ret[j] = cm_store_ca_new(ret);
-			ret[j]->cm_id = talloc_strdup(ret[j], CM_IPA_CA_NAME);
+			ret[j]->cm_busname = cm_store_ca_next_busname(ret[j]);
+			ret[j]->cm_nickname = talloc_strdup(ret[j],
+							    CM_IPA_CA_NAME);
 			ret[j]->cm_ca_type = cm_ca_external;
 			ret[j]->cm_ca_external_helper = talloc_strdup(ret[j],
 								      CM_IPA_HELPER_PATH);
@@ -1359,13 +1426,16 @@ cm_store_get_all_cas(void *parent)
 		/* Make sure we get at least one certmaster entry. */
 		for (k = 0; k < j; k++) {
 			if ((ret[k]->cm_ca_type == cm_ca_external) &&
-			    (strcmp(ret[k]->cm_id, CM_CERTMASTER_CA_NAME) == 0)) {
+			    (strcmp(ret[k]->cm_nickname,
+				    CM_CERTMASTER_CA_NAME) == 0)) {
 				break;
 			}
 		}
 		if (k == j) {
 			ret[j] = cm_store_ca_new(ret);
-			ret[j]->cm_id = talloc_strdup(ret[j], CM_CERTMASTER_CA_NAME);
+			ret[j]->cm_busname = cm_store_ca_next_busname(ret[j]);
+			ret[j]->cm_nickname = talloc_strdup(ret[j],
+							    CM_CERTMASTER_CA_NAME);
 			ret[j]->cm_ca_type = cm_ca_external;
 			ret[j]->cm_ca_external_helper = talloc_strdup(ret[j],
 								      CM_CERTMASTER_HELPER_PATH);
@@ -1379,3 +1449,96 @@ cm_store_get_all_cas(void *parent)
 	}
 	return ret;
 }
+
+struct cm_store_entry *
+cm_store_entry_dup(void *parent, struct cm_store_entry *entry)
+{
+	struct cm_store_entry *ret;
+
+	ret = talloc_ptrtype(parent, ret);
+
+	ret->cm_busname = cm_store_maybe_strdup(ret, entry->cm_busname);
+	ret->cm_store_private =
+		cm_store_maybe_strdup(ret, entry->cm_store_private);
+	ret->cm_nickname = cm_store_maybe_strdup(ret, entry->cm_nickname);
+
+	ret->cm_key_type = entry->cm_key_type;
+	ret->cm_key_storage_type = entry->cm_key_storage_type;
+	ret->cm_key_storage_location = cm_store_maybe_strdup(ret, entry->cm_key_storage_location);
+	ret->cm_key_token = cm_store_maybe_strdup(ret, entry->cm_key_token);
+	ret->cm_key_nickname = cm_store_maybe_strdup(ret, entry->cm_key_nickname);
+	ret->cm_key_pin = cm_store_maybe_strdup(ret, entry->cm_key_pin);
+	ret->cm_key_pin_file = cm_store_maybe_strdup(ret, entry->cm_key_pin_file);
+	if (ret->cm_key_pin_file != NULL) {
+		ret->cm_key_pin = NULL;
+	}
+	ret->cm_key_pubkey = cm_store_maybe_strdup(ret, entry->cm_key_pubkey);
+
+	ret->cm_cert_storage_type = entry->cm_cert_storage_type;
+	ret->cm_cert_storage_location = cm_store_maybe_strdup(ret, entry->cm_cert_storage_location);
+	ret->cm_cert_token = cm_store_maybe_strdup(ret, entry->cm_cert_token);
+	ret->cm_cert_nickname = cm_store_maybe_strdup(ret, entry->cm_cert_nickname);
+
+	ret->cm_cert_issuer = cm_store_maybe_strdup(ret, entry->cm_cert_issuer);
+	ret->cm_cert_serial = cm_store_maybe_strdup(ret, entry->cm_cert_serial);
+	ret->cm_cert_subject = cm_store_maybe_strdup(ret, entry->cm_cert_subject);
+	ret->cm_cert_spki = cm_store_maybe_strdup(ret, entry->cm_cert_spki);
+	ret->cm_cert_not_before = entry->cm_cert_not_before;
+	ret->cm_cert_not_after = entry->cm_cert_not_after;
+	ret->cm_cert_hostname = cm_store_maybe_strdupv(ret, entry->cm_cert_hostname);
+	ret->cm_cert_email = cm_store_maybe_strdupv(ret, entry->cm_cert_email);
+	ret->cm_cert_principal = cm_store_maybe_strdupv(ret, entry->cm_cert_principal);
+	ret->cm_cert_ku = cm_store_maybe_strdup(ret, entry->cm_cert_ku);
+	ret->cm_cert_eku = cm_store_maybe_strdup(ret, entry->cm_cert_eku);
+
+	ret->cm_last_expiration_check = entry->cm_last_expiration_check;
+	ret->cm_notification_method = entry->cm_notification_method;
+	ret->cm_notification_destination = cm_store_maybe_strdup(ret, entry->cm_notification_destination);
+
+	ret->cm_template_subject = cm_store_maybe_strdup(ret, entry->cm_template_subject);
+	ret->cm_template_hostname = cm_store_maybe_strdupv(ret, entry->cm_template_hostname);
+	ret->cm_template_email = cm_store_maybe_strdupv(ret, entry->cm_template_email);
+	ret->cm_template_principal = cm_store_maybe_strdupv(ret, entry->cm_template_principal);
+	ret->cm_template_ku = cm_store_maybe_strdup(ret, entry->cm_template_ku);
+	ret->cm_template_eku = cm_store_maybe_strdup(ret, entry->cm_template_eku);
+
+	ret->cm_challenge_password = cm_store_maybe_strdup(ret, entry->cm_challenge_password);
+	ret->cm_csr = cm_store_maybe_strdup(ret, entry->cm_csr);
+	ret->cm_spkac = cm_store_maybe_strdup(ret, entry->cm_spkac);
+	ret->cm_state = entry->cm_state;
+	ret->cm_autorenew = entry->cm_autorenew;
+	ret->cm_monitor = entry->cm_monitor;
+	ret->cm_ca_nickname = cm_store_maybe_strdup(ret, entry->cm_ca_nickname);
+	ret->cm_submitted = entry->cm_submitted;
+	ret->cm_ca_cookie = cm_store_maybe_strdup(ret, entry->cm_ca_cookie);
+	ret->cm_ca_error = cm_store_maybe_strdup(ret, entry->cm_ca_error);
+	ret->cm_cert = cm_store_maybe_strdup(ret, entry->cm_cert);
+	ret->cm_post_certsave_command = cm_store_maybe_strdup(ret, entry->cm_post_certsave_command);
+	ret->cm_post_certsave_uid = cm_store_maybe_strdup(ret, entry->cm_post_certsave_uid);
+
+	return ret;
+}
+
+struct cm_store_ca *
+cm_store_ca_dup(void *parent, struct cm_store_ca *ca)
+{
+	struct cm_store_ca *ret;
+
+	ret = talloc_ptrtype(parent, ret);
+	ret->cm_busname = cm_store_maybe_strdup(ret, ca->cm_busname);
+	ret->cm_store_private =
+		cm_store_maybe_strdup(ret, ca->cm_store_private);
+	ret->cm_nickname = cm_store_maybe_strdup(ret, ca->cm_nickname);
+	ret->cm_ca_known_issuer_names =
+		cm_store_maybe_strdupv(ret, ca->cm_ca_known_issuer_names);
+	ret->cm_ca_is_default = ca->cm_ca_is_default;
+	ret->cm_ca_type = ca->cm_ca_type;
+	ret->cm_ca_internal_serial =
+		cm_store_maybe_strdup(ret, ca->cm_ca_internal_serial);
+	ret->cm_ca_internal_force_issue_time =
+		ca->cm_ca_internal_force_issue_time;
+	ret->cm_ca_internal_issue_time = ca->cm_ca_internal_issue_time;
+	ret->cm_ca_external_helper =
+		cm_store_maybe_strdup(ret, ca->cm_ca_external_helper);
+	return ret;
+}
