@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2011,2012 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,6 +61,7 @@ static struct {
 	{"NEED_TO_READ_CERT", CM_NEED_TO_READ_CERT},
 	{"READING_CERT", CM_READING_CERT},
 	{"SAVED_CERT", CM_SAVED_CERT},
+	{"POST_SAVED_CERT", CM_POST_SAVED_CERT},
 	{"MONITORING", CM_MONITORING},
 	{"NEED_TO_NOTIFY", CM_NEED_TO_NOTIFY},
 	{"NOTIFYING", CM_NOTIFYING},
@@ -79,6 +80,35 @@ static struct {
 	{"NEWLY_ADDED_NEED_KEYI_READ_PIN", CM_NEWLY_ADDED_NEED_KEYINFO_READ_PIN},
 };
 
+char *
+cm_store_maybe_strdup(void *parent, const char *s)
+{
+	if ((s != NULL) && (strlen(s) > 0)) {
+		return talloc_strdup(parent, s);
+	}
+	return NULL;
+}
+
+char **
+cm_store_maybe_strdupv(void *parent, char **s)
+{
+	int i;
+	char **ret = NULL;
+	for (i = 0; (s != NULL) && (s[i] != NULL); i++) {
+		continue;
+	}
+	if (i > 0) {
+		ret = talloc_array_ptrtype(parent, ret, i + 1);
+		if (ret != NULL) {
+			for (i = 0; (s != NULL) && (s[i] != NULL); i++) {
+				ret[i] = talloc_strdup(ret, s[i]);
+			}
+			ret[i] = NULL;
+		}
+	}
+	return ret;
+}
+
 const char *
 cm_store_state_as_string(enum cm_state state)
 {
@@ -276,8 +306,7 @@ cm_store_increment_serial(void *parent, const char *old_serial)
 
 /* Produce a hex representation of the binary data. */
 char *
-cm_store_serial_from_binary(void *parent,
-			    const unsigned char *serial, int length)
+cm_store_hex_from_bin(void *parent, const unsigned char *serial, int length)
 {
 	const char *hexchars = "0123456789ABCDEF";
 	char *ret;
@@ -292,7 +321,7 @@ cm_store_serial_from_binary(void *parent,
 }
 
 /* Produce a hex representation of the hex serial number encoded as a DER
- * integer. */
+ * integer. XXX has an upper limit on the length. */
 char *
 cm_store_serial_to_der(void *parent, const char *serial)
 {
@@ -357,6 +386,9 @@ cm_store_canonicalize_directory(void *parent, const char *path)
 	} else {
 		tmp = talloc_strdup(parent, path);
 	}
+	while ((p = strstr(tmp, "/./")) != NULL) {
+		memmove(p, p + 2, strlen(p) - 1);
+	}
 	while ((p = strstr(tmp, "//")) != NULL) {
 		memmove(p, p + 1, strlen(p));
 	}
