@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,13 @@
 #include <time.h>
 
 struct cm_store_entry {
+	/* Per-instance unique identifier. */
+	char *cm_busname;
 	/* Store-private data - usually an identifier for the nonvolatile
 	 * saved copy, might be other stuff. */
 	void *cm_store_private;
-	/* A unique identifier. */
-	char *cm_id;
+	/* A persistent unique identifier or nickname. */
+	char *cm_nickname;
 	/* Type of key pair to generate [or use default settings] RSA,2048 */
 	struct cm_key_type {
 		enum cm_key_algorithm {
@@ -45,6 +47,8 @@ struct cm_store_entry {
 	char *cm_key_nickname;
 	char *cm_key_pin;
 	char *cm_key_pin_file;
+	/* Cached public key */
+	char *cm_key_pubkey;
 	/* Location of certificate [use-once default]
 	 * NSS,/etc/pki/nssdb,Server-Cert-default */
 	enum cm_cert_storage_type {
@@ -110,7 +114,7 @@ struct cm_store_entry {
 		CM_CA_REJECTED, CM_CA_WORKING,
 		CM_NEED_TO_SAVE_CERT, CM_SAVING_CERT,
 		CM_NEED_TO_READ_CERT, CM_READING_CERT,
-		CM_SAVED_CERT,
+		CM_SAVED_CERT, CM_POST_SAVED_CERT,
 		CM_MONITORING, CM_NEED_TO_NOTIFY, CM_NOTIFYING,
 		CM_NEED_GUIDANCE,
 		CM_NEWLY_ADDED,
@@ -127,7 +131,7 @@ struct cm_store_entry {
 	/* Whether to start monitoring at issue */
 	unsigned int cm_monitor:1;
 	/* Type and location of CA [or use default if NULL] */
-	char *cm_ca_name;
+	char *cm_ca_nickname;
 	/* Date of submission for in-progress submissions. */
 	time_t cm_submitted;
 	/* Value of CA cookie for in-progress submissions. */
@@ -136,14 +140,20 @@ struct cm_store_entry {
 	char *cm_ca_error;
 	/* The certificate, if we have one. */
 	char *cm_cert;
+	/* A command to run after we save the certificate. */
+	char *cm_post_certsave_command;
+	/* The UID of the user as whom we run the above command. */
+	char *cm_post_certsave_uid;
 };
 
 struct cm_store_ca {
+	/* Per-instance unique identifier. */
+	char *cm_busname;
 	/* Store-private data - usually an identifier for the nonvolatile
 	 * saved copy, might be other stuff. */
 	void *cm_store_private;
-	/* A unique identifier or nickname. */
-	char *cm_id;
+	/* A persistent unique identifier or nickname. */
+	char *cm_nickname;
 	/* A list of issuer names.  If no CA is specified when we create a new
 	 * request, and the certificate already exists and was issued by one of
 	 * these names, we'll use this CA. */
@@ -166,8 +176,10 @@ struct cm_store_ca {
 const char *cm_store_state_as_string(enum cm_state state);
 enum cm_state cm_store_state_from_string(const char *name);
 
+char *cm_store_entry_next_busname(void *parent);
 struct cm_store_entry *cm_store_files_entry_read(void *parent,
 						 const char *filename);
+char *cm_store_ca_next_busname(void *parent);
 struct cm_store_ca *cm_store_files_ca_read(void *parent,
 					   const char *filename);
 #endif
