@@ -19,10 +19,22 @@
 #define cmsubmith_h
 
 struct cm_submit_h_context;
+enum cm_submit_h_opt_negotiate {
+	cm_submit_h_negotiate_off,
+	cm_submit_h_negotiate_on
+};
+enum cm_submit_h_opt_delegate {
+	cm_submit_h_delegate_off,
+	cm_submit_h_delegate_on
+};
 struct cm_submit_h_context *cm_submit_h_init(void *parent,
 					     const char *method,
 					     const char *uri,
-					     const char *args);
+					     const char *args,
+					     const char *cainfo,
+					     const char *capath,
+					     enum cm_submit_h_opt_negotiate neg,
+					     enum cm_submit_h_opt_delegate del);
 void cm_submit_h_run(struct cm_submit_h_context *ctx);
 int cm_submit_h_result_code(struct cm_submit_h_context *ctx);
 const char *cm_submit_h_results(struct cm_submit_h_context *ctx);
