@@ -60,12 +60,19 @@ read_config_file(const char *filename)
     /* stat() the file so we know the size and can pre-allocate the right
      * amount of memory. */
     if (fstat(fd, &st) == -1) {
+	close(fd);
         cm_log(1, "Cannot stat() configuration file \"%s\": %s.\n", filename,
 	       strerror(errno));
         return NULL;
     }
     left = st.st_size;
     data = malloc(st.st_size + 1);
+    if (data == NULL) {
+	close(fd);
+        cm_log(1, "Out of memory reading configuration file \"%s\".\n",
+	       filename);
+        return NULL;
+    }
     dest = data;
     while (left != 0) {
         ssize_t res;
@@ -84,7 +91,7 @@ read_config_file(const char *filename)
         left -= res;
     }
     close(fd);
-    *dest = 0;
+    *dest = '\0';
     return data;
 }
 
